// Code generated by mockery; DO NOT EDIT.
// github.com/vektra/mockery
// template: testify

package mocks

import (
	"context"

	"github.com/meilisearch/meilisearch-go"
	mock "github.com/stretchr/testify/mock"
)

// NewMockmeilisearchDocumentReader creates a new instance of MockmeilisearchDocumentReader. It also registers a testing interface on the mock and a cleanup function to assert the mocks expectations.
// The first argument is typically a *testing.T value.
func NewMockmeilisearchDocumentReader(t interface {
	mock.TestingT
	Cleanup(func())
}) *MockmeilisearchDocumentReader {
	mock := &MockmeilisearchDocumentReader{}
	mock.Mock.Test(t)

	t.Cleanup(func() { mock.AssertExpectations(t) })

	return mock
}

// MockmeilisearchDocumentReader is an autogenerated mock type for the DocumentReader type
type MockmeilisearchDocumentReader struct {
	mock.Mock
}

type MockmeilisearchDocumentReader_Expecter struct {
	mock *mock.Mock
}

func (_m *MockmeilisearchDocumentReader) EXPECT() *MockmeilisearchDocumentReader_Expecter {
	return &MockmeilisearchDocumentReader_Expecter{mock: &_m.Mock}
}

// GetDocument provides a mock function for the type MockmeilisearchDocumentReader
func (_mock *MockmeilisearchDocumentReader) GetDocument(identifier string, request *meilisearch.DocumentQuery, documentPtr interface{}) error {
	ret := _mock.Called(identifier, request, documentPtr)

	if len(ret) == 0 {
		panic("no return value specified for GetDocument")
	}

	var r0 error
	if returnFunc, ok := ret.Get(0).(func(string, *meilisearch.DocumentQuery, interface{}) error); ok {
		r0 = returnFunc(identifier, request, documentPtr)
	} else {
		r0 = ret.Error(0)
	}
	return r0
}

// MockmeilisearchDocumentReader_GetDocument_Call is a *mock.Call that shadows Run/Return methods with type explicit version for method 'GetDocument'
type MockmeilisearchDocumentReader_GetDocument_Call struct {
	*mock.Call
}

// GetDocument is a helper method to define mock.On call
//   - identifier string
//   - request *meilisearch.DocumentQuery
//   - documentPtr interface{}
func (_e *MockmeilisearchDocumentReader_Expecter) GetDocument(identifier interface{}, request interface{}, documentPtr interface{}) *MockmeilisearchDocumentReader_GetDocument_Call {
	return &MockmeilisearchDocumentReader_GetDocument_Call{Call: _e.mock.On("GetDocument", identifier, request, documentPtr)}
}

func (_c *MockmeilisearchDocumentReader_GetDocument_Call) Run(run func(identifier string, request *meilisearch.DocumentQuery, documentPtr interface{})) *MockmeilisearchDocumentReader_GetDocument_Call {
	_c.Call.Run(func(args mock.Arguments) {
		var arg0 string
		if args[0] != nil {
			arg0 = args[0].(string)
		}
		var arg1 *meilisearch.DocumentQuery
		if args[1] != nil {
			arg1 = args[1].(*meilisearch.DocumentQuery)
		}
		var arg2 interface{}
		if args[2] != nil {
			arg2 = args[2].(interface{})
		}
		run(
			arg0,
			arg1,
			arg2,
		)
	})
	return _c
}

func (_c *MockmeilisearchDocumentReader_GetDocument_Call) Return(err error) *MockmeilisearchDocumentReader_GetDocument_Call {
	_c.Call.Return(err)
	return _c
}

func (_c *MockmeilisearchDocumentReader_GetDocument_Call) RunAndReturn(run func(identifier string, request *meilisearch.DocumentQuery, documentPtr interface{}) error) *MockmeilisearchDocumentReader_GetDocument_Call {
	_c.Call.Return(run)
	return _c
}

// GetDocumentWithContext provides a mock function for the type MockmeilisearchDocumentReader
func (_mock *MockmeilisearchDocumentReader) GetDocumentWithContext(ctx context.Context, identifier string, request *meilisearch.DocumentQuery, documentPtr interface{}) error {
	ret := _mock.Called(ctx, identifier, request, documentPtr)

	if len(ret) == 0 {
		panic("no return value specified for GetDocumentWithContext")
	}

	var r0 error
	if returnFunc, ok := ret.Get(0).(func(context.Context, string, *meilisearch.DocumentQuery, interface{}) error); ok {
		r0 = returnFunc(ctx, identifier, request, documentPtr)
	} else {
		r0 = ret.Error(0)
	}
	return r0
}

// MockmeilisearchDocumentReader_GetDocumentWithContext_Call is a *mock.Call that shadows Run/Return methods with type explicit version for method 'GetDocumentWithContext'
type MockmeilisearchDocumentReader_GetDocumentWithContext_Call struct {
	*mock.Call
}

// GetDocumentWithContext is a helper method to define mock.On call
//   - ctx context.Context
//   - identifier string
//   - request *meilisearch.DocumentQuery
//   - documentPtr interface{}
func (_e *MockmeilisearchDocumentReader_Expecter) GetDocumentWithContext(ctx interface{}, identifier interface{}, request interface{}, documentPtr interface{}) *MockmeilisearchDocumentReader_GetDocumentWithContext_Call {
	return &MockmeilisearchDocumentReader_GetDocumentWithContext_Call{Call: _e.mock.On("GetDocumentWithContext", ctx, identifier, request, documentPtr)}
}

func (_c *MockmeilisearchDocumentReader_GetDocumentWithContext_Call) Run(run func(ctx context.Context, identifier string, request *meilisearch.DocumentQuery, documentPtr interface{})) *MockmeilisearchDocumentReader_GetDocumentWithContext_Call {
	_c.Call.Run(func(args mock.Arguments) {
		var arg0 context.Context
		if args[0] != nil {
			arg0 = args[0].(context.Context)
		}
		var arg1 string
		if args[1] != nil {
			arg1 = args[1].(string)
		}
		var arg2 *meilisearch.DocumentQuery
		if args[2] != nil {
			arg2 = args[2].(*meilisearch.DocumentQuery)
		}
		var arg3 interface{}
		if args[3] != nil {
			arg3 = args[3].(interface{})
		}
		run(
			arg0,
			arg1,
			arg2,
			arg3,
		)
	})
	return _c
}

func (_c *MockmeilisearchDocumentReader_GetDocumentWithContext_Call) Return(err error) *MockmeilisearchDocumentReader_GetDocumentWithContext_Call {
	_c.Call.Return(err)
	return _c
}

func (_c *MockmeilisearchDocumentReader_GetDocumentWithContext_Call) RunAndReturn(run func(ctx context.Context, identifier string, request *meilisearch.DocumentQuery, documentPtr interface{}) error) *MockmeilisearchDocumentReader_GetDocumentWithContext_Call {
	_c.Call.Return(run)
	return _c
}

// GetDocuments provides a mock function for the type MockmeilisearchDocumentReader
func (_mock *MockmeilisearchDocumentReader) GetDocuments(param *meilisearch.DocumentsQuery, resp *meilisearch.DocumentsResult) error {
	ret := _mock.Called(param, resp)

	if len(ret) == 0 {
		panic("no return value specified for GetDocuments")
	}

	var r0 error
	if returnFunc, ok := ret.Get(0).(func(*meilisearch.DocumentsQuery, *meilisearch.DocumentsResult) error); ok {
		r0 = returnFunc(param, resp)
	} else {
		r0 = ret.Error(0)
	}
	return r0
}

// MockmeilisearchDocumentReader_GetDocuments_Call is a *mock.Call that shadows Run/Return methods with type explicit version for method 'GetDocuments'
type MockmeilisearchDocumentReader_GetDocuments_Call struct {
	*mock.Call
}

// GetDocuments is a helper method to define mock.On call
//   - param *meilisearch.DocumentsQuery
//   - resp *meilisearch.DocumentsResult
func (_e *MockmeilisearchDocumentReader_Expecter) GetDocuments(param interface{}, resp interface{}) *MockmeilisearchDocumentReader_GetDocuments_Call {
	return &MockmeilisearchDocumentReader_GetDocuments_Call{Call: _e.mock.On("GetDocuments", param, resp)}
}

func (_c *MockmeilisearchDocumentReader_GetDocuments_Call) Run(run func(param *meilisearch.DocumentsQuery, resp *meilisearch.DocumentsResult)) *MockmeilisearchDocumentReader_GetDocuments_Call {
	_c.Call.Run(func(args mock.Arguments) {
		var arg0 *meilisearch.DocumentsQuery
		if args[0] != nil {
			arg0 = args[0].(*meilisearch.DocumentsQuery)
		}
		var arg1 *meilisearch.DocumentsResult
		if args[1] != nil {
			arg1 = args[1].(*meilisearch.DocumentsResult)
		}
		run(
			arg0,
			arg1,
		)
	})
	return _c
}

func (_c *MockmeilisearchDocumentReader_GetDocuments_Call) Return(err error) *MockmeilisearchDocumentReader_GetDocuments_Call {
	_c.Call.Return(err)
	return _c
}

func (_c *MockmeilisearchDocumentReader_GetDocuments_Call) RunAndReturn(run func(param *meilisearch.DocumentsQuery, resp *meilisearch.DocumentsResult) error) *MockmeilisearchDocumentReader_GetDocuments_Call {
	_c.Call.Return(run)
	return _c
}

// GetDocumentsWithContext provides a mock function for the type MockmeilisearchDocumentReader
func (_mock *MockmeilisearchDocumentReader) GetDocumentsWithContext(ctx context.Context, param *meilisearch.DocumentsQuery, resp *meilisearch.DocumentsResult) error {
	ret := _mock.Called(ctx, param, resp)

	if len(ret) == 0 {
		panic("no return value specified for GetDocumentsWithContext")
	}

	var r0 error
	if returnFunc, ok := ret.Get(0).(func(context.Context, *meilisearch.DocumentsQuery, *meilisearch.DocumentsResult) error); ok {
		r0 = returnFunc(ctx, param, resp)
	} else {
		r0 = ret.Error(0)
	}
	return r0
}

// MockmeilisearchDocumentReader_GetDocumentsWithContext_Call is a *mock.Call that shadows Run/Return methods with type explicit version for method 'GetDocumentsWithContext'
type MockmeilisearchDocumentReader_GetDocumentsWithContext_Call struct {
	*mock.Call
}

// GetDocumentsWithContext is a helper method to define mock.On call
//   - ctx context.Context
//   - param *meilisearch.DocumentsQuery
//   - resp *meilisearch.DocumentsResult
func (_e *MockmeilisearchDocumentReader_Expecter) GetDocumentsWithContext(ctx interface{}, param interface{}, resp interface{}) *MockmeilisearchDocumentReader_GetDocumentsWithContext_Call {
	return &MockmeilisearchDocumentReader_GetDocumentsWithContext_Call{Call: _e.mock.On("GetDocumentsWithContext", ctx, param, resp)}
}

func (_c *MockmeilisearchDocumentReader_GetDocumentsWithContext_Call) Run(run func(ctx context.Context, param *meilisearch.DocumentsQuery, resp *meilisearch.DocumentsResult)) *MockmeilisearchDocumentReader_GetDocumentsWithContext_Call {
	_c.Call.Run(func(args mock.Arguments) {
		var arg0 context.Context
		if args[0] != nil {
			arg0 = args[0].(context.Context)
		}
		var arg1 *meilisearch.DocumentsQuery
		if args[1] != nil {
			arg1 = args[1].(*meilisearch.DocumentsQuery)
		}
		var arg2 *meilisearch.DocumentsResult
		if args[2] != nil {
			arg2 = args[2].(*meilisearch.DocumentsResult)
		}
		run(
			arg0,
			arg1,
			arg2,
		)
	})
	return _c
}

func (_c *MockmeilisearchDocumentReader_GetDocumentsWithContext_Call) Return(err error) *MockmeilisearchDocumentReader_GetDocumentsWithContext_Call {
	_c.Call.Return(err)
	return _c
}

func (_c *MockmeilisearchDocumentReader_GetDocumentsWithContext_Call) RunAndReturn(run func(ctx context.Context, param *meilisearch.DocumentsQuery, resp *meilisearch.DocumentsResult) error) *MockmeilisearchDocumentReader_GetDocumentsWithContext_Call {
	_c.Call.Return(run)
	return _c
}
