// Code generated by mockery; DO NOT EDIT.
// github.com/vektra/mockery
// template: testify

package mocks

import (
	"github.com/meilisearch/meilisearch-go"
	mock "github.com/stretchr/testify/mock"
)

// NewMockmeilisearchDecoder creates a new instance of MockmeilisearchDecoder. It also registers a testing interface on the mock and a cleanup function to assert the mocks expectations.
// The first argument is typically a *testing.T value.
func NewMockmeilisearchDecoder(t interface {
	mock.TestingT
	Cleanup(func())
}) *MockmeilisearchDecoder {
	mock := &MockmeilisearchDecoder{}
	mock.Mock.Test(t)

	t.Cleanup(func() { mock.AssertExpectations(t) })

	return mock
}

// MockmeilisearchDecoder is an autogenerated mock type for the Decoder type
type MockmeilisearchDecoder struct {
	mock.Mock
}

type MockmeilisearchDecoder_Expecter struct {
	mock *mock.Mock
}

func (_m *MockmeilisearchDecoder) EXPECT() *MockmeilisearchDecoder_Expecter {
	return &MockmeilisearchDecoder_Expecter{mock: &_m.Mock}
}

// Close provides a mock function for the type MockmeilisearchDecoder
func (_mock *MockmeilisearchDecoder) Close() error {
	ret := _mock.Called()

	if len(ret) == 0 {
		panic("no return value specified for Close")
	}

	var r0 error
	if returnFunc, ok := ret.Get(0).(func() error); ok {
		r0 = returnFunc()
	} else {
		r0 = ret.Error(0)
	}
	return r0
}

// MockmeilisearchDecoder_Close_Call is a *mock.Call that shadows Run/Return methods with type explicit version for method 'Close'
type MockmeilisearchDecoder_Close_Call struct {
	*mock.Call
}

// Close is a helper method to define mock.On call
func (_e *MockmeilisearchDecoder_Expecter) Close() *MockmeilisearchDecoder_Close_Call {
	return &MockmeilisearchDecoder_Close_Call{Call: _e.mock.On("Close")}
}

func (_c *MockmeilisearchDecoder_Close_Call) Run(run func()) *MockmeilisearchDecoder_Close_Call {
	_c.Call.Run(func(args mock.Arguments) {
		run()
	})
	return _c
}

func (_c *MockmeilisearchDecoder_Close_Call) Return(err error) *MockmeilisearchDecoder_Close_Call {
	_c.Call.Return(err)
	return _c
}

func (_c *MockmeilisearchDecoder_Close_Call) RunAndReturn(run func() error) *MockmeilisearchDecoder_Close_Call {
	_c.Call.Return(run)
	return _c
}

// Err provides a mock function for the type MockmeilisearchDecoder
func (_mock *MockmeilisearchDecoder) Err() error {
	ret := _mock.Called()

	if len(ret) == 0 {
		panic("no return value specified for Err")
	}

	var r0 error
	if returnFunc, ok := ret.Get(0).(func() error); ok {
		r0 = returnFunc()
	} else {
		r0 = ret.Error(0)
	}
	return r0
}

// MockmeilisearchDecoder_Err_Call is a *mock.Call that shadows Run/Return methods with type explicit version for method 'Err'
type MockmeilisearchDecoder_Err_Call struct {
	*mock.Call
}

// Err is a helper method to define mock.On call
func (_e *MockmeilisearchDecoder_Expecter) Err() *MockmeilisearchDecoder_Err_Call {
	return &MockmeilisearchDecoder_Err_Call{Call: _e.mock.On("Err")}
}

func (_c *MockmeilisearchDecoder_Err_Call) Run(run func()) *MockmeilisearchDecoder_Err_Call {
	_c.Call.Run(func(args mock.Arguments) {
		run()
	})
	return _c
}

func (_c *MockmeilisearchDecoder_Err_Call) Return(err error) *MockmeilisearchDecoder_Err_Call {
	_c.Call.Return(err)
	return _c
}

func (_c *MockmeilisearchDecoder_Err_Call) RunAndReturn(run func() error) *MockmeilisearchDecoder_Err_Call {
	_c.Call.Return(run)
	return _c
}

// Event provides a mock function for the type MockmeilisearchDecoder
func (_mock *MockmeilisearchDecoder) Event() meilisearch.Event {
	ret := _mock.Called()

	if len(ret) == 0 {
		panic("no return value specified for Event")
	}

	var r0 meilisearch.Event
	if returnFunc, ok := ret.Get(0).(func() meilisearch.Event); ok {
		r0 = returnFunc()
	} else {
		r0 = ret.Get(0).(meilisearch.Event)
	}
	return r0
}

// MockmeilisearchDecoder_Event_Call is a *mock.Call that shadows Run/Return methods with type explicit version for method 'Event'
type MockmeilisearchDecoder_Event_Call struct {
	*mock.Call
}

// Event is a helper method to define mock.On call
func (_e *MockmeilisearchDecoder_Expecter) Event() *MockmeilisearchDecoder_Event_Call {
	return &MockmeilisearchDecoder_Event_Call{Call: _e.mock.On("Event")}
}

func (_c *MockmeilisearchDecoder_Event_Call) Run(run func()) *MockmeilisearchDecoder_Event_Call {
	_c.Call.Run(func(args mock.Arguments) {
		run()
	})
	return _c
}

func (_c *MockmeilisearchDecoder_Event_Call) Return(event meilisearch.Event) *MockmeilisearchDecoder_Event_Call {
	_c.Call.Return(event)
	return _c
}

func (_c *MockmeilisearchDecoder_Event_Call) RunAndReturn(run func() meilisearch.Event) *MockmeilisearchDecoder_Event_Call {
	_c.Call.Return(run)
	return _c
}

// Next provides a mock function for the type MockmeilisearchDecoder
func (_mock *MockmeilisearchDecoder) Next() bool {
	ret := _mock.Called()

	if len(ret) == 0 {
		panic("no return value specified for Next")
	}

	var r0 bool
	if returnFunc, ok := ret.Get(0).(func() bool); ok {
		r0 = returnFunc()
	} else {
		r0 = ret.Get(0).(bool)
	}
	return r0
}

// MockmeilisearchDecoder_Next_Call is a *mock.Call that shadows Run/Return methods with type explicit version for method 'Next'
type MockmeilisearchDecoder_Next_Call struct {
	*mock.Call
}

// Next is a helper method to define mock.On call
func (_e *MockmeilisearchDecoder_Expecter) Next() *MockmeilisearchDecoder_Next_Call {
	return &MockmeilisearchDecoder_Next_Call{Call: _e.mock.On("Next")}
}

func (_c *MockmeilisearchDecoder_Next_Call) Run(run func()) *MockmeilisearchDecoder_Next_Call {
	_c.Call.Run(func(args mock.Arguments) {
		run()
	})
	return _c
}

func (_c *MockmeilisearchDecoder_Next_Call) Return(b bool) *MockmeilisearchDecoder_Next_Call {
	_c.Call.Return(b)
	return _c
}

func (_c *MockmeilisearchDecoder_Next_Call) RunAndReturn(run func() bool) *MockmeilisearchDecoder_Next_Call {
	_c.Call.Return(run)
	return _c
}
