// Code generated by mockery; DO NOT EDIT.
// github.com/vektra/mockery
// template: testify

package mocks

import (
	"context"
	"time"

	"github.com/meilisearch/meilisearch-go"
	mock "github.com/stretchr/testify/mock"
)

// NewMockmeilisearchTaskManager creates a new instance of MockmeilisearchTaskManager. It also registers a testing interface on the mock and a cleanup function to assert the mocks expectations.
// The first argument is typically a *testing.T value.
func NewMockmeilisearchTaskManager(t interface {
	mock.TestingT
	Cleanup(func())
}) *MockmeilisearchTaskManager {
	mock := &MockmeilisearchTaskManager{}
	mock.Mock.Test(t)

	t.Cleanup(func() { mock.AssertExpectations(t) })

	return mock
}

// MockmeilisearchTaskManager is an autogenerated mock type for the TaskManager type
type MockmeilisearchTaskManager struct {
	mock.Mock
}

type MockmeilisearchTaskManager_Expecter struct {
	mock *mock.Mock
}

func (_m *MockmeilisearchTaskManager) EXPECT() *MockmeilisearchTaskManager_Expecter {
	return &MockmeilisearchTaskManager_Expecter{mock: &_m.Mock}
}

// CancelTasks provides a mock function for the type MockmeilisearchTaskManager
func (_mock *MockmeilisearchTaskManager) CancelTasks(param *meilisearch.CancelTasksQuery) (*meilisearch.TaskInfo, error) {
	ret := _mock.Called(param)

	if len(ret) == 0 {
		panic("no return value specified for CancelTasks")
	}

	var r0 *meilisearch.TaskInfo
	var r1 error
	if returnFunc, ok := ret.Get(0).(func(*meilisearch.CancelTasksQuery) (*meilisearch.TaskInfo, error)); ok {
		return returnFunc(param)
	}
	if returnFunc, ok := ret.Get(0).(func(*meilisearch.CancelTasksQuery) *meilisearch.TaskInfo); ok {
		r0 = returnFunc(param)
	} else {
		if ret.Get(0) != nil {
			r0 = ret.Get(0).(*meilisearch.TaskInfo)
		}
	}
	if returnFunc, ok := ret.Get(1).(func(*meilisearch.CancelTasksQuery) error); ok {
		r1 = returnFunc(param)
	} else {
		r1 = ret.Error(1)
	}
	return r0, r1
}

// MockmeilisearchTaskManager_CancelTasks_Call is a *mock.Call that shadows Run/Return methods with type explicit version for method 'CancelTasks'
type MockmeilisearchTaskManager_CancelTasks_Call struct {
	*mock.Call
}

// CancelTasks is a helper method to define mock.On call
//   - param *meilisearch.CancelTasksQuery
func (_e *MockmeilisearchTaskManager_Expecter) CancelTasks(param interface{}) *MockmeilisearchTaskManager_CancelTasks_Call {
	return &MockmeilisearchTaskManager_CancelTasks_Call{Call: _e.mock.On("CancelTasks", param)}
}

func (_c *MockmeilisearchTaskManager_CancelTasks_Call) Run(run func(param *meilisearch.CancelTasksQuery)) *MockmeilisearchTaskManager_CancelTasks_Call {
	_c.Call.Run(func(args mock.Arguments) {
		var arg0 *meilisearch.CancelTasksQuery
		if args[0] != nil {
			arg0 = args[0].(*meilisearch.CancelTasksQuery)
		}
		run(
			arg0,
		)
	})
	return _c
}

func (_c *MockmeilisearchTaskManager_CancelTasks_Call) Return(taskInfo *meilisearch.TaskInfo, err error) *MockmeilisearchTaskManager_CancelTasks_Call {
	_c.Call.Return(taskInfo, err)
	return _c
}

func (_c *MockmeilisearchTaskManager_CancelTasks_Call) RunAndReturn(run func(param *meilisearch.CancelTasksQuery) (*meilisearch.TaskInfo, error)) *MockmeilisearchTaskManager_CancelTasks_Call {
	_c.Call.Return(run)
	return _c
}

// CancelTasksWithContext provides a mock function for the type MockmeilisearchTaskManager
func (_mock *MockmeilisearchTaskManager) CancelTasksWithContext(ctx context.Context, param *meilisearch.CancelTasksQuery) (*meilisearch.TaskInfo, error) {
	ret := _mock.Called(ctx, param)

	if len(ret) == 0 {
		panic("no return value specified for CancelTasksWithContext")
	}

	var r0 *meilisearch.TaskInfo
	var r1 error
	if returnFunc, ok := ret.Get(0).(func(context.Context, *meilisearch.CancelTasksQuery) (*meilisearch.TaskInfo, error)); ok {
		return returnFunc(ctx, param)
	}
	if returnFunc, ok := ret.Get(0).(func(context.Context, *meilisearch.CancelTasksQuery) *meilisearch.TaskInfo); ok {
		r0 = returnFunc(ctx, param)
	} else {
		if ret.Get(0) != nil {
			r0 = ret.Get(0).(*meilisearch.TaskInfo)
		}
	}
	if returnFunc, ok := ret.Get(1).(func(context.Context, *meilisearch.CancelTasksQuery) error); ok {
		r1 = returnFunc(ctx, param)
	} else {
		r1 = ret.Error(1)
	}
	return r0, r1
}

// MockmeilisearchTaskManager_CancelTasksWithContext_Call is a *mock.Call that shadows Run/Return methods with type explicit version for method 'CancelTasksWithContext'
type MockmeilisearchTaskManager_CancelTasksWithContext_Call struct {
	*mock.Call
}

// CancelTasksWithContext is a helper method to define mock.On call
//   - ctx context.Context
//   - param *meilisearch.CancelTasksQuery
func (_e *MockmeilisearchTaskManager_Expecter) CancelTasksWithContext(ctx interface{}, param interface{}) *MockmeilisearchTaskManager_CancelTasksWithContext_Call {
	return &MockmeilisearchTaskManager_CancelTasksWithContext_Call{Call: _e.mock.On("CancelTasksWithContext", ctx, param)}
}

func (_c *MockmeilisearchTaskManager_CancelTasksWithContext_Call) Run(run func(ctx context.Context, param *meilisearch.CancelTasksQuery)) *MockmeilisearchTaskManager_CancelTasksWithContext_Call {
	_c.Call.Run(func(args mock.Arguments) {
		var arg0 context.Context
		if args[0] != nil {
			arg0 = args[0].(context.Context)
		}
		var arg1 *meilisearch.CancelTasksQuery
		if args[1] != nil {
			arg1 = args[1].(*meilisearch.CancelTasksQuery)
		}
		run(
			arg0,
			arg1,
		)
	})
	return _c
}

func (_c *MockmeilisearchTaskManager_CancelTasksWithContext_Call) Return(taskInfo *meilisearch.TaskInfo, err error) *MockmeilisearchTaskManager_CancelTasksWithContext_Call {
	_c.Call.Return(taskInfo, err)
	return _c
}

func (_c *MockmeilisearchTaskManager_CancelTasksWithContext_Call) RunAndReturn(run func(ctx context.Context, param *meilisearch.CancelTasksQuery) (*meilisearch.TaskInfo, error)) *MockmeilisearchTaskManager_CancelTasksWithContext_Call {
	_c.Call.Return(run)
	return _c
}

// DeleteTasks provides a mock function for the type MockmeilisearchTaskManager
func (_mock *MockmeilisearchTaskManager) DeleteTasks(param *meilisearch.DeleteTasksQuery) (*meilisearch.TaskInfo, error) {
	ret := _mock.Called(param)

	if len(ret) == 0 {
		panic("no return value specified for DeleteTasks")
	}

	var r0 *meilisearch.TaskInfo
	var r1 error
	if returnFunc, ok := ret.Get(0).(func(*meilisearch.DeleteTasksQuery) (*meilisearch.TaskInfo, error)); ok {
		return returnFunc(param)
	}
	if returnFunc, ok := ret.Get(0).(func(*meilisearch.DeleteTasksQuery) *meilisearch.TaskInfo); ok {
		r0 = returnFunc(param)
	} else {
		if ret.Get(0) != nil {
			r0 = ret.Get(0).(*meilisearch.TaskInfo)
		}
	}
	if returnFunc, ok := ret.Get(1).(func(*meilisearch.DeleteTasksQuery) error); ok {
		r1 = returnFunc(param)
	} else {
		r1 = ret.Error(1)
	}
	return r0, r1
}

// MockmeilisearchTaskManager_DeleteTasks_Call is a *mock.Call that shadows Run/Return methods with type explicit version for method 'DeleteTasks'
type MockmeilisearchTaskManager_DeleteTasks_Call struct {
	*mock.Call
}

// DeleteTasks is a helper method to define mock.On call
//   - param *meilisearch.DeleteTasksQuery
func (_e *MockmeilisearchTaskManager_Expecter) DeleteTasks(param interface{}) *MockmeilisearchTaskManager_DeleteTasks_Call {
	return &MockmeilisearchTaskManager_DeleteTasks_Call{Call: _e.mock.On("DeleteTasks", param)}
}

func (_c *MockmeilisearchTaskManager_DeleteTasks_Call) Run(run func(param *meilisearch.DeleteTasksQuery)) *MockmeilisearchTaskManager_DeleteTasks_Call {
	_c.Call.Run(func(args mock.Arguments) {
		var arg0 *meilisearch.DeleteTasksQuery
		if args[0] != nil {
			arg0 = args[0].(*meilisearch.DeleteTasksQuery)
		}
		run(
			arg0,
		)
	})
	return _c
}

func (_c *MockmeilisearchTaskManager_DeleteTasks_Call) Return(taskInfo *meilisearch.TaskInfo, err error) *MockmeilisearchTaskManager_DeleteTasks_Call {
	_c.Call.Return(taskInfo, err)
	return _c
}

func (_c *MockmeilisearchTaskManager_DeleteTasks_Call) RunAndReturn(run func(param *meilisearch.DeleteTasksQuery) (*meilisearch.TaskInfo, error)) *MockmeilisearchTaskManager_DeleteTasks_Call {
	_c.Call.Return(run)
	return _c
}

// DeleteTasksWithContext provides a mock function for the type MockmeilisearchTaskManager
func (_mock *MockmeilisearchTaskManager) DeleteTasksWithContext(ctx context.Context, param *meilisearch.DeleteTasksQuery) (*meilisearch.TaskInfo, error) {
	ret := _mock.Called(ctx, param)

	if len(ret) == 0 {
		panic("no return value specified for DeleteTasksWithContext")
	}

	var r0 *meilisearch.TaskInfo
	var r1 error
	if returnFunc, ok := ret.Get(0).(func(context.Context, *meilisearch.DeleteTasksQuery) (*meilisearch.TaskInfo, error)); ok {
		return returnFunc(ctx, param)
	}
	if returnFunc, ok := ret.Get(0).(func(context.Context, *meilisearch.DeleteTasksQuery) *meilisearch.TaskInfo); ok {
		r0 = returnFunc(ctx, param)
	} else {
		if ret.Get(0) != nil {
			r0 = ret.Get(0).(*meilisearch.TaskInfo)
		}
	}
	if returnFunc, ok := ret.Get(1).(func(context.Context, *meilisearch.DeleteTasksQuery) error); ok {
		r1 = returnFunc(ctx, param)
	} else {
		r1 = ret.Error(1)
	}
	return r0, r1
}

// MockmeilisearchTaskManager_DeleteTasksWithContext_Call is a *mock.Call that shadows Run/Return methods with type explicit version for method 'DeleteTasksWithContext'
type MockmeilisearchTaskManager_DeleteTasksWithContext_Call struct {
	*mock.Call
}

// DeleteTasksWithContext is a helper method to define mock.On call
//   - ctx context.Context
//   - param *meilisearch.DeleteTasksQuery
func (_e *MockmeilisearchTaskManager_Expecter) DeleteTasksWithContext(ctx interface{}, param interface{}) *MockmeilisearchTaskManager_DeleteTasksWithContext_Call {
	return &MockmeilisearchTaskManager_DeleteTasksWithContext_Call{Call: _e.mock.On("DeleteTasksWithContext", ctx, param)}
}

func (_c *MockmeilisearchTaskManager_DeleteTasksWithContext_Call) Run(run func(ctx context.Context, param *meilisearch.DeleteTasksQuery)) *MockmeilisearchTaskManager_DeleteTasksWithContext_Call {
	_c.Call.Run(func(args mock.Arguments) {
		var arg0 context.Context
		if args[0] != nil {
			arg0 = args[0].(context.Context)
		}
		var arg1 *meilisearch.DeleteTasksQuery
		if args[1] != nil {
			arg1 = args[1].(*meilisearch.DeleteTasksQuery)
		}
		run(
			arg0,
			arg1,
		)
	})
	return _c
}

func (_c *MockmeilisearchTaskManager_DeleteTasksWithContext_Call) Return(taskInfo *meilisearch.TaskInfo, err error) *MockmeilisearchTaskManager_DeleteTasksWithContext_Call {
	_c.Call.Return(taskInfo, err)
	return _c
}

func (_c *MockmeilisearchTaskManager_DeleteTasksWithContext_Call) RunAndReturn(run func(ctx context.Context, param *meilisearch.DeleteTasksQuery) (*meilisearch.TaskInfo, error)) *MockmeilisearchTaskManager_DeleteTasksWithContext_Call {
	_c.Call.Return(run)
	return _c
}

// GetTask provides a mock function for the type MockmeilisearchTaskManager
func (_mock *MockmeilisearchTaskManager) GetTask(taskUID int64) (*meilisearch.Task, error) {
	ret := _mock.Called(taskUID)

	if len(ret) == 0 {
		panic("no return value specified for GetTask")
	}

	var r0 *meilisearch.Task
	var r1 error
	if returnFunc, ok := ret.Get(0).(func(int64) (*meilisearch.Task, error)); ok {
		return returnFunc(taskUID)
	}
	if returnFunc, ok := ret.Get(0).(func(int64) *meilisearch.Task); ok {
		r0 = returnFunc(taskUID)
	} else {
		if ret.Get(0) != nil {
			r0 = ret.Get(0).(*meilisearch.Task)
		}
	}
	if returnFunc, ok := ret.Get(1).(func(int64) error); ok {
		r1 = returnFunc(taskUID)
	} else {
		r1 = ret.Error(1)
	}
	return r0, r1
}

// MockmeilisearchTaskManager_GetTask_Call is a *mock.Call that shadows Run/Return methods with type explicit version for method 'GetTask'
type MockmeilisearchTaskManager_GetTask_Call struct {
	*mock.Call
}

// GetTask is a helper method to define mock.On call
//   - taskUID int64
func (_e *MockmeilisearchTaskManager_Expecter) GetTask(taskUID interface{}) *MockmeilisearchTaskManager_GetTask_Call {
	return &MockmeilisearchTaskManager_GetTask_Call{Call: _e.mock.On("GetTask", taskUID)}
}

func (_c *MockmeilisearchTaskManager_GetTask_Call) Run(run func(taskUID int64)) *MockmeilisearchTaskManager_GetTask_Call {
	_c.Call.Run(func(args mock.Arguments) {
		var arg0 int64
		if args[0] != nil {
			arg0 = args[0].(int64)
		}
		run(
			arg0,
		)
	})
	return _c
}

func (_c *MockmeilisearchTaskManager_GetTask_Call) Return(task *meilisearch.Task, err error) *MockmeilisearchTaskManager_GetTask_Call {
	_c.Call.Return(task, err)
	return _c
}

func (_c *MockmeilisearchTaskManager_GetTask_Call) RunAndReturn(run func(taskUID int64) (*meilisearch.Task, error)) *MockmeilisearchTaskManager_GetTask_Call {
	_c.Call.Return(run)
	return _c
}

// GetTaskDocuments provides a mock function for the type MockmeilisearchTaskManager
func (_mock *MockmeilisearchTaskManager) GetTaskDocuments(taskUID int64, dst interface{}) error {
	ret := _mock.Called(taskUID, dst)

	if len(ret) == 0 {
		panic("no return value specified for GetTaskDocuments")
	}

	var r0 error
	if returnFunc, ok := ret.Get(0).(func(int64, interface{}) error); ok {
		r0 = returnFunc(taskUID, dst)
	} else {
		r0 = ret.Error(0)
	}
	return r0
}

// MockmeilisearchTaskManager_GetTaskDocuments_Call is a *mock.Call that shadows Run/Return methods with type explicit version for method 'GetTaskDocuments'
type MockmeilisearchTaskManager_GetTaskDocuments_Call struct {
	*mock.Call
}

// GetTaskDocuments is a helper method to define mock.On call
//   - taskUID int64
//   - dst interface{}
func (_e *MockmeilisearchTaskManager_Expecter) GetTaskDocuments(taskUID interface{}, dst interface{}) *MockmeilisearchTaskManager_GetTaskDocuments_Call {
	return &MockmeilisearchTaskManager_GetTaskDocuments_Call{Call: _e.mock.On("GetTaskDocuments", taskUID, dst)}
}

func (_c *MockmeilisearchTaskManager_GetTaskDocuments_Call) Run(run func(taskUID int64, dst interface{})) *MockmeilisearchTaskManager_GetTaskDocuments_Call {
	_c.Call.Run(func(args mock.Arguments) {
		var arg0 int64
		if args[0] != nil {
			arg0 = args[0].(int64)
		}
		var arg1 interface{}
		if args[1] != nil {
			arg1 = args[1].(interface{})
		}
		run(
			arg0,
			arg1,
		)
	})
	return _c
}

func (_c *MockmeilisearchTaskManager_GetTaskDocuments_Call) Return(err error) *MockmeilisearchTaskManager_GetTaskDocuments_Call {
	_c.Call.Return(err)
	return _c
}

func (_c *MockmeilisearchTaskManager_GetTaskDocuments_Call) RunAndReturn(run func(taskUID int64, dst interface{}) error) *MockmeilisearchTaskManager_GetTaskDocuments_Call {
	_c.Call.Return(run)
	return _c
}

// GetTaskDocumentsWithContext provides a mock function for the type MockmeilisearchTaskManager
func (_mock *MockmeilisearchTaskManager) GetTaskDocumentsWithContext(ctx context.Context, taskUID int64, dst interface{}) error {
	ret := _mock.Called(ctx, taskUID, dst)

	if len(ret) == 0 {
		panic("no return value specified for GetTaskDocumentsWithContext")
	}

	var r0 error
	if returnFunc, ok := ret.Get(0).(func(context.Context, int64, interface{}) error); ok {
		r0 = returnFunc(ctx, taskUID, dst)
	} else {
		r0 = ret.Error(0)
	}
	return r0
}

// MockmeilisearchTaskManager_GetTaskDocumentsWithContext_Call is a *mock.Call that shadows Run/Return methods with type explicit version for method 'GetTaskDocumentsWithContext'
type MockmeilisearchTaskManager_GetTaskDocumentsWithContext_Call struct {
	*mock.Call
}

// GetTaskDocumentsWithContext is a helper method to define mock.On call
//   - ctx context.Context
//   - taskUID int64
//   - dst interface{}
func (_e *MockmeilisearchTaskManager_Expecter) GetTaskDocumentsWithContext(ctx interface{}, taskUID interface{}, dst interface{}) *MockmeilisearchTaskManager_GetTaskDocumentsWithContext_Call {
	return &MockmeilisearchTaskManager_GetTaskDocumentsWithContext_Call{Call: _e.mock.On("GetTaskDocumentsWithContext", ctx, taskUID, dst)}
}

func (_c *MockmeilisearchTaskManager_GetTaskDocumentsWithContext_Call) Run(run func(ctx context.Context, taskUID int64, dst interface{})) *MockmeilisearchTaskManager_GetTaskDocumentsWithContext_Call {
	_c.Call.Run(func(args mock.Arguments) {
		var arg0 context.Context
		if args[0] != nil {
			arg0 = args[0].(context.Context)
		}
		var arg1 int64
		if args[1] != nil {
			arg1 = args[1].(int64)
		}
		var arg2 interface{}
		if args[2] != nil {
			arg2 = args[2].(interface{})
		}
		run(
			arg0,
			arg1,
			arg2,
		)
	})
	return _c
}

func (_c *MockmeilisearchTaskManager_GetTaskDocumentsWithContext_Call) Return(err error) *MockmeilisearchTaskManager_GetTaskDocumentsWithContext_Call {
	_c.Call.Return(err)
	return _c
}

func (_c *MockmeilisearchTaskManager_GetTaskDocumentsWithContext_Call) RunAndReturn(run func(ctx context.Context, taskUID int64, dst interface{}) error) *MockmeilisearchTaskManager_GetTaskDocumentsWithContext_Call {
	_c.Call.Return(run)
	return _c
}

// GetTaskWithContext provides a mock function for the type MockmeilisearchTaskManager
func (_mock *MockmeilisearchTaskManager) GetTaskWithContext(ctx context.Context, taskUID int64) (*meilisearch.Task, error) {
	ret := _mock.Called(ctx, taskUID)

	if len(ret) == 0 {
		panic("no return value specified for GetTaskWithContext")
	}

	var r0 *meilisearch.Task
	var r1 error
	if returnFunc, ok := ret.Get(0).(func(context.Context, int64) (*meilisearch.Task, error)); ok {
		return returnFunc(ctx, taskUID)
	}
	if returnFunc, ok := ret.Get(0).(func(context.Context, int64) *meilisearch.Task); ok {
		r0 = returnFunc(ctx, taskUID)
	} else {
		if ret.Get(0) != nil {
			r0 = ret.Get(0).(*meilisearch.Task)
		}
	}
	if returnFunc, ok := ret.Get(1).(func(context.Context, int64) error); ok {
		r1 = returnFunc(ctx, taskUID)
	} else {
		r1 = ret.Error(1)
	}
	return r0, r1
}

// MockmeilisearchTaskManager_GetTaskWithContext_Call is a *mock.Call that shadows Run/Return methods with type explicit version for method 'GetTaskWithContext'
type MockmeilisearchTaskManager_GetTaskWithContext_Call struct {
	*mock.Call
}

// GetTaskWithContext is a helper method to define mock.On call
//   - ctx context.Context
//   - taskUID int64
func (_e *MockmeilisearchTaskManager_Expecter) GetTaskWithContext(ctx interface{}, taskUID interface{}) *MockmeilisearchTaskManager_GetTaskWithContext_Call {
	return &MockmeilisearchTaskManager_GetTaskWithContext_Call{Call: _e.mock.On("GetTaskWithContext", ctx, taskUID)}
}

func (_c *MockmeilisearchTaskManager_GetTaskWithContext_Call) Run(run func(ctx context.Context, taskUID int64)) *MockmeilisearchTaskManager_GetTaskWithContext_Call {
	_c.Call.Run(func(args mock.Arguments) {
		var arg0 context.Context
		if args[0] != nil {
			arg0 = args[0].(context.Context)
		}
		var arg1 int64
		if args[1] != nil {
			arg1 = args[1].(int64)
		}
		run(
			arg0,
			arg1,
		)
	})
	return _c
}

func (_c *MockmeilisearchTaskManager_GetTaskWithContext_Call) Return(task *meilisearch.Task, err error) *MockmeilisearchTaskManager_GetTaskWithContext_Call {
	_c.Call.Return(task, err)
	return _c
}

func (_c *MockmeilisearchTaskManager_GetTaskWithContext_Call) RunAndReturn(run func(ctx context.Context, taskUID int64) (*meilisearch.Task, error)) *MockmeilisearchTaskManager_GetTaskWithContext_Call {
	_c.Call.Return(run)
	return _c
}

// GetTasks provides a mock function for the type MockmeilisearchTaskManager
func (_mock *MockmeilisearchTaskManager) GetTasks(param *meilisearch.TasksQuery) (*meilisearch.TaskResult, error) {
	ret := _mock.Called(param)

	if len(ret) == 0 {
		panic("no return value specified for GetTasks")
	}

	var r0 *meilisearch.TaskResult
	var r1 error
	if returnFunc, ok := ret.Get(0).(func(*meilisearch.TasksQuery) (*meilisearch.TaskResult, error)); ok {
		return returnFunc(param)
	}
	if returnFunc, ok := ret.Get(0).(func(*meilisearch.TasksQuery) *meilisearch.TaskResult); ok {
		r0 = returnFunc(param)
	} else {
		if ret.Get(0) != nil {
			r0 = ret.Get(0).(*meilisearch.TaskResult)
		}
	}
	if returnFunc, ok := ret.Get(1).(func(*meilisearch.TasksQuery) error); ok {
		r1 = returnFunc(param)
	} else {
		r1 = ret.Error(1)
	}
	return r0, r1
}

// MockmeilisearchTaskManager_GetTasks_Call is a *mock.Call that shadows Run/Return methods with type explicit version for method 'GetTasks'
type MockmeilisearchTaskManager_GetTasks_Call struct {
	*mock.Call
}

// GetTasks is a helper method to define mock.On call
//   - param *meilisearch.TasksQuery
func (_e *MockmeilisearchTaskManager_Expecter) GetTasks(param interface{}) *MockmeilisearchTaskManager_GetTasks_Call {
	return &MockmeilisearchTaskManager_GetTasks_Call{Call: _e.mock.On("GetTasks", param)}
}

func (_c *MockmeilisearchTaskManager_GetTasks_Call) Run(run func(param *meilisearch.TasksQuery)) *MockmeilisearchTaskManager_GetTasks_Call {
	_c.Call.Run(func(args mock.Arguments) {
		var arg0 *meilisearch.TasksQuery
		if args[0] != nil {
			arg0 = args[0].(*meilisearch.TasksQuery)
		}
		run(
			arg0,
		)
	})
	return _c
}

func (_c *MockmeilisearchTaskManager_GetTasks_Call) Return(taskResult *meilisearch.TaskResult, err error) *MockmeilisearchTaskManager_GetTasks_Call {
	_c.Call.Return(taskResult, err)
	return _c
}

func (_c *MockmeilisearchTaskManager_GetTasks_Call) RunAndReturn(run func(param *meilisearch.TasksQuery) (*meilisearch.TaskResult, error)) *MockmeilisearchTaskManager_GetTasks_Call {
	_c.Call.Return(run)
	return _c
}

// GetTasksWithContext provides a mock function for the type MockmeilisearchTaskManager
func (_mock *MockmeilisearchTaskManager) GetTasksWithContext(ctx context.Context, param *meilisearch.TasksQuery) (*meilisearch.TaskResult, error) {
	ret := _mock.Called(ctx, param)

	if len(ret) == 0 {
		panic("no return value specified for GetTasksWithContext")
	}

	var r0 *meilisearch.TaskResult
	var r1 error
	if returnFunc, ok := ret.Get(0).(func(context.Context, *meilisearch.TasksQuery) (*meilisearch.TaskResult, error)); ok {
		return returnFunc(ctx, param)
	}
	if returnFunc, ok := ret.Get(0).(func(context.Context, *meilisearch.TasksQuery) *meilisearch.TaskResult); ok {
		r0 = returnFunc(ctx, param)
	} else {
		if ret.Get(0) != nil {
			r0 = ret.Get(0).(*meilisearch.TaskResult)
		}
	}
	if returnFunc, ok := ret.Get(1).(func(context.Context, *meilisearch.TasksQuery) error); ok {
		r1 = returnFunc(ctx, param)
	} else {
		r1 = ret.Error(1)
	}
	return r0, r1
}

// MockmeilisearchTaskManager_GetTasksWithContext_Call is a *mock.Call that shadows Run/Return methods with type explicit version for method 'GetTasksWithContext'
type MockmeilisearchTaskManager_GetTasksWithContext_Call struct {
	*mock.Call
}

// GetTasksWithContext is a helper method to define mock.On call
//   - ctx context.Context
//   - param *meilisearch.TasksQuery
func (_e *MockmeilisearchTaskManager_Expecter) GetTasksWithContext(ctx interface{}, param interface{}) *MockmeilisearchTaskManager_GetTasksWithContext_Call {
	return &MockmeilisearchTaskManager_GetTasksWithContext_Call{Call: _e.mock.On("GetTasksWithContext", ctx, param)}
}

func (_c *MockmeilisearchTaskManager_GetTasksWithContext_Call) Run(run func(ctx context.Context, param *meilisearch.TasksQuery)) *MockmeilisearchTaskManager_GetTasksWithContext_Call {
	_c.Call.Run(func(args mock.Arguments) {
		var arg0 context.Context
		if args[0] != nil {
			arg0 = args[0].(context.Context)
		}
		var arg1 *meilisearch.TasksQuery
		if args[1] != nil {
			arg1 = args[1].(*meilisearch.TasksQuery)
		}
		run(
			arg0,
			arg1,
		)
	})
	return _c
}

func (_c *MockmeilisearchTaskManager_GetTasksWithContext_Call) Return(taskResult *meilisearch.TaskResult, err error) *MockmeilisearchTaskManager_GetTasksWithContext_Call {
	_c.Call.Return(taskResult, err)
	return _c
}

func (_c *MockmeilisearchTaskManager_GetTasksWithContext_Call) RunAndReturn(run func(ctx context.Context, param *meilisearch.TasksQuery) (*meilisearch.TaskResult, error)) *MockmeilisearchTaskManager_GetTasksWithContext_Call {
	_c.Call.Return(run)
	return _c
}

// WaitForTask provides a mock function for the type MockmeilisearchTaskManager
func (_mock *MockmeilisearchTaskManager) WaitForTask(taskUID int64, interval time.Duration) (*meilisearch.Task, error) {
	ret := _mock.Called(taskUID, interval)

	if len(ret) == 0 {
		panic("no return value specified for WaitForTask")
	}

	var r0 *meilisearch.Task
	var r1 error
	if returnFunc, ok := ret.Get(0).(func(int64, time.Duration) (*meilisearch.Task, error)); ok {
		return returnFunc(taskUID, interval)
	}
	if returnFunc, ok := ret.Get(0).(func(int64, time.Duration) *meilisearch.Task); ok {
		r0 = returnFunc(taskUID, interval)
	} else {
		if ret.Get(0) != nil {
			r0 = ret.Get(0).(*meilisearch.Task)
		}
	}
	if returnFunc, ok := ret.Get(1).(func(int64, time.Duration) error); ok {
		r1 = returnFunc(taskUID, interval)
	} else {
		r1 = ret.Error(1)
	}
	return r0, r1
}

// MockmeilisearchTaskManager_WaitForTask_Call is a *mock.Call that shadows Run/Return methods with type explicit version for method 'WaitForTask'
type MockmeilisearchTaskManager_WaitForTask_Call struct {
	*mock.Call
}

// WaitForTask is a helper method to define mock.On call
//   - taskUID int64
//   - interval time.Duration
func (_e *MockmeilisearchTaskManager_Expecter) WaitForTask(taskUID interface{}, interval interface{}) *MockmeilisearchTaskManager_WaitForTask_Call {
	return &MockmeilisearchTaskManager_WaitForTask_Call{Call: _e.mock.On("WaitForTask", taskUID, interval)}
}

func (_c *MockmeilisearchTaskManager_WaitForTask_Call) Run(run func(taskUID int64, interval time.Duration)) *MockmeilisearchTaskManager_WaitForTask_Call {
	_c.Call.Run(func(args mock.Arguments) {
		var arg0 int64
		if args[0] != nil {
			arg0 = args[0].(int64)
		}
		var arg1 time.Duration
		if args[1] != nil {
			arg1 = args[1].(time.Duration)
		}
		run(
			arg0,
			arg1,
		)
	})
	return _c
}

func (_c *MockmeilisearchTaskManager_WaitForTask_Call) Return(task *meilisearch.Task, err error) *MockmeilisearchTaskManager_WaitForTask_Call {
	_c.Call.Return(task, err)
	return _c
}

func (_c *MockmeilisearchTaskManager_WaitForTask_Call) RunAndReturn(run func(taskUID int64, interval time.Duration) (*meilisearch.Task, error)) *MockmeilisearchTaskManager_WaitForTask_Call {
	_c.Call.Return(run)
	return _c
}

// WaitForTaskWithContext provides a mock function for the type MockmeilisearchTaskManager
func (_mock *MockmeilisearchTaskManager) WaitForTaskWithContext(ctx context.Context, taskUID int64, interval time.Duration) (*meilisearch.Task, error) {
	ret := _mock.Called(ctx, taskUID, interval)

	if len(ret) == 0 {
		panic("no return value specified for WaitForTaskWithContext")
	}

	var r0 *meilisearch.Task
	var r1 error
	if returnFunc, ok := ret.Get(0).(func(context.Context, int64, time.Duration) (*meilisearch.Task, error)); ok {
		return returnFunc(ctx, taskUID, interval)
	}
	if returnFunc, ok := ret.Get(0).(func(context.Context, int64, time.Duration) *meilisearch.Task); ok {
		r0 = returnFunc(ctx, taskUID, interval)
	} else {
		if ret.Get(0) != nil {
			r0 = ret.Get(0).(*meilisearch.Task)
		}
	}
	if returnFunc, ok := ret.Get(1).(func(context.Context, int64, time.Duration) error); ok {
		r1 = returnFunc(ctx, taskUID, interval)
	} else {
		r1 = ret.Error(1)
	}
	return r0, r1
}

// MockmeilisearchTaskManager_WaitForTaskWithContext_Call is a *mock.Call that shadows Run/Return methods with type explicit version for method 'WaitForTaskWithContext'
type MockmeilisearchTaskManager_WaitForTaskWithContext_Call struct {
	*mock.Call
}

// WaitForTaskWithContext is a helper method to define mock.On call
//   - ctx context.Context
//   - taskUID int64
//   - interval time.Duration
func (_e *MockmeilisearchTaskManager_Expecter) WaitForTaskWithContext(ctx interface{}, taskUID interface{}, interval interface{}) *MockmeilisearchTaskManager_WaitForTaskWithContext_Call {
	return &MockmeilisearchTaskManager_WaitForTaskWithContext_Call{Call: _e.mock.On("WaitForTaskWithContext", ctx, taskUID, interval)}
}

func (_c *MockmeilisearchTaskManager_WaitForTaskWithContext_Call) Run(run func(ctx context.Context, taskUID int64, interval time.Duration)) *MockmeilisearchTaskManager_WaitForTaskWithContext_Call {
	_c.Call.Run(func(args mock.Arguments) {
		var arg0 context.Context
		if args[0] != nil {
			arg0 = args[0].(context.Context)
		}
		var arg1 int64
		if args[1] != nil {
			arg1 = args[1].(int64)
		}
		var arg2 time.Duration
		if args[2] != nil {
			arg2 = args[2].(time.Duration)
		}
		run(
			arg0,
			arg1,
			arg2,
		)
	})
	return _c
}

func (_c *MockmeilisearchTaskManager_WaitForTaskWithContext_Call) Return(task *meilisearch.Task, err error) *MockmeilisearchTaskManager_WaitForTaskWithContext_Call {
	_c.Call.Return(task, err)
	return _c
}

func (_c *MockmeilisearchTaskManager_WaitForTaskWithContext_Call) RunAndReturn(run func(ctx context.Context, taskUID int64, interval time.Duration) (*meilisearch.Task, error)) *MockmeilisearchTaskManager_WaitForTaskWithContext_Call {
	_c.Call.Return(run)
	return _c
}
